@@ -9,6 +9,7 @@ import (
 	"sync"
 
 	"github.com/xtls/xray-core/common/dice"
+	"github.com/xtls/xray-core/common/errors"
 	"github.com/xtls/xray-core/common/protocol"
 )
 
@@ -21,9 +22,7 @@ type Validator struct {
 	behaviorFused bool
 }
 
-var (
-	ErrNotFound = newError("Not Found")
-)
+var ErrNotFound = errors.New("Not Found")
 
 // Add a Shadowsocks user.
 func (v *Validator) Add(u *protocol.MemoryUser) error {
@@ -32,7 +31,7 @@ func (v *Validator) Add(u *protocol.MemoryUser) error {
 
 	account := u.Account.(*MemoryAccount)
 	if !account.Cipher.IsAEAD() && len(v.users) > 0 {
-		return newError("The cipher is not support Single-port Multi-user")
+		return errors.New("The cipher is not support Single-port Multi-user")
 	}
 	v.users = append(v.users, u)
 
@@ -48,7 +47,7 @@ func (v *Validator) Add(u *protocol.MemoryUser) error {
 // Del a Shadowsocks user with a non-empty Email.
 func (v *Validator) Del(email string) error {
 	if email == "" {
-		return newError("Email must not be empty.")
+		return errors.New("Email must not be empty.")
 	}
 
 	v.Lock()
@@ -64,7 +63,7 @@ func (v *Validator) Del(email string) error {
 	}
 
 	if idx == -1 {
-		return newError("User ", email, " not found.")
+		return errors.New("User ", email, " not found.")
 	}
 	ulen := len(v.users)
 
@@ -75,6 +74,40 @@ func (v *Validator) Del(email string) error {
 	return nil
 }
 
+// GetByEmail Get a Shadowsocks user with a non-empty Email.
+func (v *Validator) GetByEmail(email string) *protocol.MemoryUser {
+	if email == "" {
+		return nil
+	}
+
+	v.Lock()
+	defer v.Unlock()
+
+	email = strings.ToLower(email)
+	for _, u := range v.users {
+		if strings.EqualFold(u.Email, email) {
+			return u
+		}
+	}
+	return nil
+}
+
+// GetAll get all users
+func (v *Validator) GetAll() []*protocol.MemoryUser {
+	v.Lock()
+	defer v.Unlock()
+	dst := make([]*protocol.MemoryUser, len(v.users))
+	copy(dst, v.users)
+	return dst
+}
+
+// GetCount get users count
+func (v *Validator) GetCount() int64 {
+	v.Lock()
+	defer v.Unlock()
+	return int64(len(v.users))
+}
+
 // Get a Shadowsocks user.
 func (v *Validator) Get(bs []byte, command protocol.RequestCommand) (u *protocol.MemoryUser, aead cipher.AEAD, ret []byte, ivLen int32, err error) {
 	v.RLock()
@@ -82,6 +115,11 @@ func (v *Validator) Get(bs []byte, command protocol.RequestCommand) (u *protocol
 
 	for _, user := range v.users {
 		if account := user.Account.(*MemoryAccount); account.Cipher.IsAEAD() {
+			// AEAD payload decoding requires the payload to be over 32 bytes
+			if len(bs) < 32 {
+				continue
+			}
+
 			aeadCipher := account.Cipher.(*AEADCipher)
 			ivLen = aeadCipher.IVSize()
 			iv := bs[:ivLen]
